@@ -33,7 +33,6 @@
 #include "debug.h"
 #include <vte/vte.h>
 #include "vteaccess.h"
-#include "vteint.h"
 #include "vte-private.h"
 
 #ifdef HAVE_LOCALE_H
@@ -41,6 +40,12 @@
 #endif
 #include <glib/gi18n-lib.h>
 
+#define TERMINAL_FROM_ACCESSIBLE(a) (VTE_TERMINAL(gtk_accessible_get_widget(GTK_ACCESSIBLE(a))))
+
+#define IMPL(t) (_vte_terminal_get_impl(t))
+#define IMPL_FROM_WIDGET(w) (IMPL(VTE_TERMINAL(w)))
+#define IMPL_FROM_ACCESSIBLE(a) (IMPL_FROM_WIDGET(gtk_accessible_get_widget(GTK_ACCESSIBLE(a))))
+
 enum {
         ACTION_MENU,
         LAST_ACTION
@@ -142,13 +147,6 @@ xy_from_offset (VteTerminalAccessiblePrivate *priv,
 	*y = cur_y;
 }
 
-/* "Oh yeah, that's selected.  Sure." callback. */
-static gboolean
-all_selected(VteTerminal *terminal, glong column, glong row, gpointer data)
-{
-	return TRUE;
-}
-
 static void
 emit_text_caret_moved(GObject *object, glong caret)
 {
@@ -204,9 +202,8 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
                                                       GArray **old_characters)
 {
 	VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private(accessible);
-        VteTerminal *terminal;
 	struct _VteCharAttributes attrs;
-	char *next, *tmp;
+	char *next;
 	long row, offset, caret;
 	long ccol, crow;
 	guint i;
@@ -237,7 +234,8 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
 	}
 
 	/* Re-read the contents of the widget if the contents have changed. */
-	terminal = VTE_TERMINAL(gtk_accessible_get_widget(GTK_ACCESSIBLE(accessible)));
+        VteTerminal* terminal = TERMINAL_FROM_ACCESSIBLE(accessible);
+        auto impl = IMPL(terminal);
 	if (priv->snapshot_contents_invalid) {
 		/* Free the outdated snapshot data, unless the caller
 		 * wants it. */
@@ -285,17 +283,9 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
 		priv->snapshot_linebreaks = g_array_new(FALSE, FALSE, sizeof(int));
 
 		/* Get a new view of the uber-label. */
-		tmp = vte_terminal_get_text_include_trailing_spaces(terminal,
-								    all_selected,
-								    NULL,
-								    priv->snapshot_attributes);
-		if (tmp == NULL) {
-			/* Aaargh!  We're screwed. */
-			return;
-		}
-		priv->snapshot_text = g_string_new_len(tmp,
-						       priv->snapshot_attributes->len);
-		g_free(tmp);
+		priv->snapshot_text = impl->get_text_displayed_a11y(true /* wrap */,
+                                                                             true /* include trailing whitespace */,
+                                                                             priv->snapshot_attributes);
 
 		/* Get the offsets to the beginnings of each character. */
 		i = 0;
@@ -906,14 +896,13 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 {
         VteTerminalAccessible *accessible = VTE_TERMINAL_ACCESSIBLE(text);
 	VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private(accessible);
-        VteTerminal *terminal;
 	gunichar current, prev, next;
 	guint start, end, line;
 
 	vte_terminal_accessible_update_private_data_if_needed(accessible,
 							      NULL, NULL);
 
-        terminal = VTE_TERMINAL(gtk_accessible_get_widget (GTK_ACCESSIBLE(text)));
+        auto impl = IMPL_FROM_ACCESSIBLE(text);
 
 	_vte_debug_print(VTE_DEBUG_ALLY,
 			"Getting %s %s at %d of %d.\n",
@@ -947,7 +936,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			/* Back up to the previous non-word-word transition. */
 			while (offset > 0) {
 				prev = vte_terminal_accessible_get_character_at_offset(text, offset - 1);
-				if (_vte_terminal_is_word_char(terminal, prev)) {
+				if (impl->is_word_char(prev)) {
 					offset--;
 				} else {
 					break;
@@ -961,7 +950,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			if (direction == direction_previous) {
 				while (offset > 0) {
 					prev = vte_terminal_accessible_get_character_at_offset(text, offset - 1);
-					if (!_vte_terminal_is_word_char(terminal, prev)) {
+					if (!impl->is_word_char(prev)) {
 						offset--;
 					} else {
 						break;
@@ -969,7 +958,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 				}
 				while (offset > 0) {
 					prev = vte_terminal_accessible_get_character_at_offset(text, offset - 1);
-					if (_vte_terminal_is_word_char(terminal, prev)) {
+					if (impl->is_word_char(prev)) {
 						offset--;
 					} else {
 						break;
@@ -984,7 +973,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			if (direction == direction_next) {
 				while (offset < (int) priv->snapshot_characters->len) {
 					next = vte_terminal_accessible_get_character_at_offset(text, offset);
-					if (_vte_terminal_is_word_char(terminal, next)) {
+					if (impl->is_word_char(next)) {
 						offset++;
 					} else {
 						break;
@@ -992,7 +981,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 				}
 				while (offset < (int) priv->snapshot_characters->len) {
 					next = vte_terminal_accessible_get_character_at_offset(text, offset);
-					if (!_vte_terminal_is_word_char(terminal, next)) {
+					if (!impl->is_word_char(next)) {
 						offset++;
 					} else {
 						break;
@@ -1003,7 +992,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			/* Now find the end of this word. */
 			while (offset < (int) priv->snapshot_characters->len) {
 				current = vte_terminal_accessible_get_character_at_offset(text, offset);
-				if (_vte_terminal_is_word_char(terminal, current)) {
+				if (impl->is_word_char(current)) {
 					offset++;
 				} else {
 					break;
@@ -1013,7 +1002,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			/* Now find the next non-word-word transition */
 			while (offset < (int) priv->snapshot_characters->len) {
 				next = vte_terminal_accessible_get_character_at_offset(text, offset);
-				if (!_vte_terminal_is_word_char(terminal, next)) {
+				if (!impl->is_word_char(next)) {
 					offset++;
 				} else {
 					break;
@@ -1026,8 +1015,8 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			current = vte_terminal_accessible_get_character_at_offset(text, offset);
 			while (offset > 0) {
 				prev = vte_terminal_accessible_get_character_at_offset(text, offset - 1);
-				if (_vte_terminal_is_word_char(terminal, prev) &&
-				    !_vte_terminal_is_word_char(terminal, current)) {
+				if (impl->is_word_char(prev) &&
+				    !impl->is_word_char(current)) {
 					break;
 				} else {
 					offset--;
@@ -1042,7 +1031,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			if (direction == direction_previous) {
 				while (offset > 0) {
 					prev = vte_terminal_accessible_get_character_at_offset(text, offset - 1);
-					if (_vte_terminal_is_word_char(terminal, prev)) {
+					if (impl->is_word_char(prev)) {
 						offset--;
 					} else {
 						break;
@@ -1051,8 +1040,8 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 				current = vte_terminal_accessible_get_character_at_offset(text, offset);
 				while (offset > 0) {
 					prev = vte_terminal_accessible_get_character_at_offset(text, offset - 1);
-					if (_vte_terminal_is_word_char(terminal, prev) &&
-					    !_vte_terminal_is_word_char(terminal, current)) {
+					if (impl->is_word_char(prev) &&
+					    !impl->is_word_char(current)) {
 						break;
 					} else {
 						offset--;
@@ -1068,7 +1057,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			if (direction == direction_next) {
 				while (offset < (int) priv->snapshot_characters->len) {
 					current = vte_terminal_accessible_get_character_at_offset(text, offset);
-					if (!_vte_terminal_is_word_char(terminal, current)) {
+					if (!impl->is_word_char(current)) {
 						offset++;
 					} else {
 						break;
@@ -1076,7 +1065,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 				}
 				while (offset < (int) priv->snapshot_characters->len) {
 					current = vte_terminal_accessible_get_character_at_offset(text, offset);
-					if (_vte_terminal_is_word_char(terminal, current)) {
+					if (impl->is_word_char(current)) {
 						offset++;
 					} else {
 						break;
@@ -1087,7 +1076,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			/* Now find the next word end. */
 			while (offset < (int) priv->snapshot_characters->len) {
 				current = vte_terminal_accessible_get_character_at_offset(text, offset);
-				if (!_vte_terminal_is_word_char(terminal, current)) {
+				if (!impl->is_word_char(current)) {
 					offset++;
 				} else {
 					break;
@@ -1095,7 +1084,7 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			}
 			while (offset < (int) priv->snapshot_characters->len) {
 				current = vte_terminal_accessible_get_character_at_offset(text, offset);
-				if (_vte_terminal_is_word_char(terminal, current)) {
+				if (impl->is_word_char(current)) {
 					offset++;
 				} else {
 					break;
@@ -1489,8 +1478,9 @@ vte_terminal_accessible_remove_selection(AtkText *text,
 	}
 
 	terminal = VTE_TERMINAL (widget);
+        auto impl = IMPL_FROM_WIDGET(widget);
 	if (selection_number == 0 && vte_terminal_get_has_selection (terminal)) {
-		_vte_terminal_remove_selection (terminal);
+		impl->deselect_all();
 		return TRUE;
 	} else {
 		return FALSE;
@@ -1514,11 +1504,12 @@ vte_terminal_accessible_set_selection(AtkText *text, gint selection_number,
 	}
 
 	terminal = VTE_TERMINAL (widget);
+        auto impl = IMPL_FROM_WIDGET(widget);
 	if (selection_number != 0) {
 		return FALSE;
 	}
 	if (vte_terminal_get_has_selection (terminal)) {
-		_vte_terminal_remove_selection (terminal);
+		impl->deselect_all();
 	}
 
 	return vte_terminal_accessible_add_selection (text, start_offset, end_offset);
