@@ -150,7 +150,7 @@ button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer data)
 	int tag;
 	GtkBorder padding;
         gboolean has_extra_match;
-        char *extra_match;
+        char *extra_match = NULL;
 
 	switch (event->button) {
 	case 3:
@@ -574,7 +574,7 @@ add_dingus (VteTerminal *terminal,
                                               &error);
                 else
 #endif
-                        gregex = g_regex_new(dingus[i], G_REGEX_OPTIMIZE, 0, &error);
+                        gregex = g_regex_new(dingus[i], G_REGEX_OPTIMIZE | G_REGEX_MULTILINE, 0, &error);
 
                 if (error) {
                         g_warning("Failed to compile regex '%s': %s\n",
@@ -614,7 +614,8 @@ main(int argc, char **argv)
 		(char *) "FOO=BAR", (char *) "BOO=BIZ",
 #endif
 		NULL};
-        char *transparent = NULL;
+        int transparency_percent = 0;
+        gboolean no_argb_visual = FALSE;
         char *encoding = NULL;
         char *cjk_ambiguous_width = NULL;
 	gboolean audible = FALSE,
@@ -636,7 +637,8 @@ main(int argc, char **argv)
 	char *cursor_shape_string = NULL;
 	char *scrollbar_policy_string = NULL;
         char *border_width_string = NULL;
-        char *cursor_color_string = NULL;
+        char *cursor_foreground_color_string = NULL;
+        char *cursor_background_color_string = NULL;
         char *highlight_foreground_color_string = NULL;
         char *highlight_background_color_string = NULL;
         char **dingus = NULL;
@@ -676,8 +678,8 @@ main(int argc, char **argv)
 		},
 		{
 			"transparent", 'T', 0,
-			G_OPTION_ARG_STRING, &transparent,
-			"Enable the use of a transparent background", "ALPHA"
+			G_OPTION_ARG_INT, &transparency_percent,
+                        "Enable the use of a transparent background", "0..100",
 		},
 		{
 			"double-buffer", '2', G_OPTION_FLAG_REVERSE,
@@ -741,9 +743,14 @@ main(int argc, char **argv)
 			"Cursor blink mode (system|on|off)", "MODE"
 		},
 		{
-			"cursor-color", 0, 0,
-			G_OPTION_ARG_STRING, &cursor_color_string,
-			"Enable a colored cursor", NULL
+			"cursor-background-color", 0, 0,
+			G_OPTION_ARG_STRING, &cursor_background_color_string,
+			"Enable a colored cursor background", NULL
+		},
+		{
+			"cursor-foreground-color", 0, 0,
+			G_OPTION_ARG_STRING, &cursor_foreground_color_string,
+			"Enable a colored cursor foreground", NULL
 		},
 		{
 			"cursor-shape", 0, 0,
@@ -771,6 +778,12 @@ main(int argc, char **argv)
 			G_OPTION_ARG_NONE, &reverse,
 			"Reverse foreground/background colors", NULL
 		},
+		{
+			"no-argb-visual", 0, 0,
+			G_OPTION_ARG_NONE, &no_argb_visual,
+                        "Don't use an ARGB visual",
+			NULL
+		},
 		{
 			"no-geometry-hints", 'G', G_OPTION_FLAG_REVERSE,
 			G_OPTION_ARG_NONE, &use_geometry_hints,
@@ -868,6 +881,13 @@ main(int argc, char **argv)
 
 	gdk_window_set_debug_updates(debug);
 
+        g_object_set(gtk_settings_get_default(),
+                     "gtk-enable-mnemonics", FALSE,
+                     "gtk-enable-accels",FALSE,
+                     /* Make gtk+ CSD not steal F10 from the terminal */
+                     "gtk-menu-bar-accel", NULL,
+                     NULL);
+
 	/* Create a window to hold the scrolling shell, and hook its
 	 * delete event to the quit function.. */
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -882,10 +902,17 @@ main(int argc, char **argv)
         }
 
 	/* Set ARGB visual */
-	screen = gtk_widget_get_screen (window);
-	visual = gdk_screen_get_rgba_visual(screen);
-	if (visual)
-		gtk_widget_set_visual(GTK_WIDGET(window), visual);
+        if (transparency_percent != 0) {
+                if (!no_argb_visual) {
+                        screen = gtk_widget_get_screen (window);
+                        visual = gdk_screen_get_rgba_visual(screen);
+                        if (visual)
+                                gtk_widget_set_visual(GTK_WIDGET(window), visual);
+                }
+
+                /* Without this transparency doesn't work; see bug #729884. */
+                gtk_widget_set_app_paintable(GTK_WIDGET(window), TRUE);
+        }
 
 	if (use_scrolled_window) {
 		scrolled_window = gtk_scrolled_window_new (NULL, NULL);
@@ -975,18 +1002,23 @@ main(int argc, char **argv)
 	vte_terminal_set_scrollback_lines(terminal, lines);
 	vte_terminal_set_mouse_autohide(terminal, TRUE);
 
-	if (transparent != NULL) {
-                back.alpha = g_ascii_strtod (transparent, NULL);
-                g_free (transparent);
+	if (transparency_percent != 0) {
+                back.alpha = (double)(100 - CLAMP(transparency_percent, 0, 100)) / 100.;
         }
 
 	vte_terminal_set_colors(terminal, &fore, &back, NULL, 0);
 
-	if (cursor_color_string) {
+	if (cursor_foreground_color_string) {
                 GdkRGBA rgba;
-                if (parse_color (cursor_color_string, &rgba))
+                if (parse_color (cursor_foreground_color_string, &rgba))
+                        vte_terminal_set_color_cursor_foreground(terminal, &rgba);
+                g_free(cursor_foreground_color_string);
+	}
+	if (cursor_background_color_string) {
+                GdkRGBA rgba;
+                if (parse_color (cursor_background_color_string, &rgba))
                         vte_terminal_set_color_cursor(terminal, &rgba);
-                g_free(cursor_color_string);
+                g_free(cursor_background_color_string);
 	}
 	if (highlight_foreground_color_string) {
                 GdkRGBA rgba;
