@@ -24,11 +24,21 @@
 #include <gtk/gtk.h>
 #include <glib.h>
 #include "debug.h"
-#include "vtedraw.h"
+#include "vtedraw.hh"
 #include "vte-private.h"
 
 #include <pango/pangocairo.h>
 
+#define VTE_DRAW_SINGLE_WIDE_CHARACTERS	\
+					" !\"#$%&'()*+,-./" \
+					"0123456789" \
+					":;<=>?@" \
+					"ABCDEFGHIJKLMNOPQRSTUVWXYZ" \
+					"[\\]^_`" \
+					"abcdefghijklmnopqrstuvwxyz" \
+					"{|}~" \
+					""
+
 
 /* Overview:
  *
@@ -661,7 +671,7 @@ font_info_get_unistr_info (struct font_info *info,
 
 	g_string_set_size (info->string, 0);
 	_vte_unistr_append_to_string (c, info->string);
-	pango_layout_set_text (info->layout, info->string->str, -1);
+	pango_layout_set_text (info->layout, info->string->str, info->string->len);
 	pango_layout_get_extents (info->layout, NULL, &logical);
 
 	uinfo->width = PANGO_PIXELS_CEIL (logical.width);
@@ -732,10 +742,7 @@ guint _vte_draw_get_style(gboolean bold, gboolean italic) {
 }
 
 struct _vte_draw {
-	GtkWidget *widget;
-
 	struct font_info *fonts[4];
-	cairo_pattern_t *bg_pattern;
 
 	cairo_t *cr;
 };
@@ -753,23 +760,12 @@ _vte_draw_new (void)
 	return draw;
 }
 
-cairo_t *
-_vte_draw_get_context (struct _vte_draw *draw)
-{
-  return draw->cr;
-}
-
 void
 _vte_draw_free (struct _vte_draw *draw)
 {
 	gint style;
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_free\n");
 
-	if (draw->bg_pattern != NULL) {
-		cairo_pattern_destroy (draw->bg_pattern);
-		draw->bg_pattern = NULL;
-	}
-
 	/* Free all fonts (make sure to destroy every font only once)*/
 	for (style = 3; style >= 0; style--) {
 		if (draw->fonts[style] != NULL &&
@@ -797,31 +793,30 @@ _vte_draw_set_cairo (struct _vte_draw *draw,
         }
 }
 
-void
-_vte_draw_set_background_solid(struct _vte_draw *draw,
-                               const GdkRGBA *color)
+static void
+_vte_draw_set_source_color_alpha (struct _vte_draw *draw,
+                                  vte::color::rgb const* color,
+                                  double alpha)
 {
-	if (draw->bg_pattern)
-		cairo_pattern_destroy (draw->bg_pattern);
-
-	draw->bg_pattern = cairo_pattern_create_rgba (color->red,
-						      color->green,
-						      color->blue,
-						      color->alpha);
+        g_assert(draw->cr);
+	cairo_set_source_rgba (draw->cr,
+			      color->red / 65535.,
+			      color->green / 65535.,
+			      color->blue / 65535.,
+			      alpha);
 }
 
 void
-_vte_draw_clear (struct _vte_draw *draw, gint x, gint y, gint width, gint height)
+_vte_draw_clear (struct _vte_draw *draw, gint x, gint y, gint width, gint height,
+                 vte::color::rgb const* color, double alpha)
 {
-	g_return_if_fail (draw->bg_pattern != NULL);
-
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_clear (%d, %d, %d, %d)\n",
 			  x,y,width, height);
 
         g_assert(draw->cr);
 	cairo_rectangle (draw->cr, x, y, width, height);
 	cairo_set_operator (draw->cr, CAIRO_OPERATOR_SOURCE);
-	cairo_set_source (draw->cr, draw->bg_pattern);
+	_vte_draw_set_source_color_alpha(draw, color, alpha);
 	cairo_fill (draw->cr);
 }
 
@@ -923,19 +918,6 @@ _vte_draw_has_bold (struct _vte_draw *draw, guint style)
 	return (draw->fonts[style ^ VTE_DRAW_BOLD] != draw->fonts[style]);
 }
 
-static void
-_vte_draw_set_source_color_alpha (struct _vte_draw *draw,
-                                  const PangoColor *color,
-                                  guchar            alpha)
-{
-        g_assert(draw->cr);
-	cairo_set_source_rgba (draw->cr,
-			      color->red / 65535.,
-			      color->green / 65535.,
-			      color->blue / 65535.,
-			      alpha / 255.);
-}
-
 /* Check if a unicode character is actually a graphic character we draw
  * ourselves to handle cases where fonts don't have glyphs for them. */
 static gboolean
@@ -950,7 +932,7 @@ _vte_draw_unichar_is_local_graphic(vteunistr c)
 /* Draw the graphic representation of a line-drawing or special graphics
  * character. */
 static void
-_vte_draw_terminal_draw_graphic(struct _vte_draw *draw, vteunistr c, const PangoColor *fg,
+_vte_draw_terminal_draw_graphic(struct _vte_draw *draw, vteunistr c, vte::color::rgb const* fg,
                                 gint x, gint y,
                                 gint column_width, gint columns, gint row_height)
 {
@@ -1410,7 +1392,7 @@ _vte_draw_terminal_draw_graphic(struct _vte_draw *draw, vteunistr c, const Pango
 static void
 _vte_draw_text_internal (struct _vte_draw *draw,
 			 struct _vte_draw_text_request *requests, gsize n_requests,
-			 const PangoColor *color, guchar alpha, guint style)
+			 vte::color::rgb const* color, double alpha, guint style)
 {
 	gsize i;
 	cairo_scaled_font_t *last_scaled_font = NULL;
@@ -1484,7 +1466,7 @@ _vte_draw_text_internal (struct _vte_draw *draw,
 void
 _vte_draw_text (struct _vte_draw *draw,
 	       struct _vte_draw_text_request *requests, gsize n_requests,
-	       const PangoColor *color, guchar alpha, guint style)
+	       vte::color::rgb const* color, double alpha, guint style)
 {
         g_assert(draw->cr);
 
@@ -1496,7 +1478,7 @@ _vte_draw_text (struct _vte_draw *draw,
 			g_string_append_unichar (string, requests[n].c);
 		}
 		str = g_string_free (string, FALSE);
-		g_printerr ("draw_text (\"%s\", len=%" G_GSIZE_FORMAT ", color=(%d,%d,%d,%d), %s - %s)\n",
+		g_printerr ("draw_text (\"%s\", len=%" G_GSIZE_FORMAT ", color=(%d,%d,%d,%.3f), %s - %s)\n",
 				str, n_requests, color->red, color->green, color->blue, alpha,
 				(style & VTE_DRAW_BOLD)   ? "bold"   : "normal",
 				(style & VTE_DRAW_ITALIC) ? "italic" : "regular");
@@ -1543,12 +1525,12 @@ _vte_draw_has_char (struct _vte_draw *draw, vteunistr c, guint style)
 gboolean
 _vte_draw_char (struct _vte_draw *draw,
 	       struct _vte_draw_text_request *request,
-	       const PangoColor *color, guchar alpha, guint style)
+	       vte::color::rgb const* color, double alpha, guint style)
 {
 	gboolean has_char;
 
 	_vte_debug_print (VTE_DEBUG_DRAW,
-			"draw_char ('%c', color=(%d,%d,%d,%d), %s, %s)\n",
+			"draw_char ('%c', color=(%d,%d,%d,%.3f), %s, %s)\n",
 			request->c,
 			color->red, color->green, color->blue,
 			alpha,
@@ -1565,12 +1547,12 @@ _vte_draw_char (struct _vte_draw *draw,
 void
 _vte_draw_draw_rectangle (struct _vte_draw *draw,
 			 gint x, gint y, gint width, gint height,
-			 const PangoColor *color, guchar alpha)
+			 vte::color::rgb const* color, double alpha)
 {
         g_assert(draw->cr);
 
 	_vte_debug_print (VTE_DEBUG_DRAW,
-			"draw_rectangle (%d, %d, %d, %d, color=(%d,%d,%d,%d))\n",
+			"draw_rectangle (%d, %d, %d, %d, color=(%d,%d,%d,%.3f))\n",
 			x,y,width,height,
 			color->red, color->green, color->blue,
 			alpha);
@@ -1585,12 +1567,12 @@ _vte_draw_draw_rectangle (struct _vte_draw *draw,
 void
 _vte_draw_fill_rectangle (struct _vte_draw *draw,
 			 gint x, gint y, gint width, gint height,
-			 const PangoColor *color, guchar alpha)
+			 vte::color::rgb const* color, double alpha)
 {
         g_assert(draw->cr);
 
 	_vte_debug_print (VTE_DEBUG_DRAW,
-			"draw_fill_rectangle (%d, %d, %d, %d, color=(%d,%d,%d,%d))\n",
+			"draw_fill_rectangle (%d, %d, %d, %d, color=(%d,%d,%d,%.3f))\n",
 			x,y,width,height,
 			color->red, color->green, color->blue,
 			alpha);
@@ -1600,3 +1582,16 @@ _vte_draw_fill_rectangle (struct _vte_draw *draw,
 	_vte_draw_set_source_color_alpha (draw, color, alpha);
 	cairo_fill (draw->cr);
 }
+
+
+void
+_vte_draw_draw_line(struct _vte_draw *draw,
+                    gint x, gint y, gint xp, gint yp,
+                    int line_width,
+                    vte::color::rgb const *color, double alpha)
+{
+	_vte_draw_fill_rectangle(draw,
+                                 x, y,
+                                 MAX(line_width, xp - x + 1), MAX(line_width, yp - y + 1),
+                                 color, alpha);
+}
